#***********************************************************************
# Licensed Material - Property of IBM
#
# 5724-M24
#
# (c) Copyright IBM Corp. 2009.  All rights reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#************************************************************************

from java.lang import System
import sys

#Constants for Boolean types
True = 1 == 1
False = 1 == 0


TRUE = 1
FALSE = 0

####################################################
##
## Check if the appName exists.
##
####################################################

def isAppExist(appName):
	try :
		AdminApp.listModules(appName)
	except :
		print "*** ",appName,"is not installed."
		return 0
	print  "*** ",appName,"is installed."
	return 1


####################################################
##
## Check if the $adminModule exists in the $appName module.
##
####################################################

def isAdminModuleExist (appName ,appModule) :
	try :
		apps = AdminApp.listModules(appName)
	except : 
		print "Application ",appName," does not exists."
		raise	
	
	if(apps.find(appModule)>0) :
		print "module",appModule," exists in the application ", appName
		return 1
	
	print "module",appModule," does not exists in the application ", appName	
	return 0


################################
##
## Delete the admin module war file
##
################################
def deleteAdminModule(appName,adminModule) :
	try :
		# delete the module inside the application
		AdminApp.update(appName, 'modulefile', '[-operation delete -contenturi '+adminModule+']')
	except :
		print "Error while deleting the existing "+adminModule+" module"
		raise
			
		
			
################################
##
## Deploy the admin module war file
##
################################
def deployAdminModule (appName, adminModule, WASInstallRoot, inNodeName, contextRoot) :
	
	appExist =  isAppExist(appName)
	
	# if application does not exist, return without deploy.
	if (appExist == 0) :
		print "The application"+ appName+" is not installed."
		return
	
	
	adminModuleExist = isAdminModuleExist(appName,adminModule)
	if (adminModuleExist == 1) :
	         deleteAdminModule(appName,adminModule)
	         
	
	warfile = WASInstallRoot+"/systemApps/"+appName+".ear/"+adminModule
	
	
	nodesList = AdminConfig.list("Node").split(java.lang.System.getProperty("line.separator"))
	print nodesList
	for node in  nodesList :
		nodeName = AdminConfig.showAttribute(node,"name")
		if (nodeName == inNodeName) :
			serversList = AdminConfig.list("Server",node).split(java.lang.System.getProperty("line.separator"))
			
			for server in serversList :
				serverName = AdminConfig.showAttribute(server ,"name")
                                serverType = AdminConfig.showAttribute(server ,"serverType")
				if(serverType  == "APPLICATION_SERVER"):
				    try :
					
					print "updating",serverName 
					AdminApp.update(appName, 'modulefile', '[-operation add -contents ' + '"' + warfile + '"' + ' -node '+nodeName+ ' -server '+serverName+' -contenturi '+adminModule+' -custom paavalidation=true -usedefaultbindings -contextroot '+contextRoot+' -usedefaultbindings -MapWebModToVH [[.* .* admin_host]]]')
					AdminConfig.save()                   			
				    except :
					print "Could not deploy "+adminModule+" in "+appName+" due to error"
					
                        
                        print "Completed deployment of ",adminModule," in "+appName+" module."
                        	


#------------------------------------------------------------------------------------------
# Utility to handle list
#------------------------------------------------------------------------------------------

def wsadminToList(inStr):
	outList = []
	if (len(inStr) > 0 and inStr[0] == '[' and inStr[ - 1] == ']'):
		if(inStr[1: - 1].find("\"") >= 0):
			tmpList = splitWithQuotes(inStr[1: - 1]) #splits space/quotes seperated lists
		else:
			tmpList = inStr[1: - 1].split(' ') #splits space-separated lists,		
	else:
		tmpList = inStr.split("\n")   #splits for Windows or Linux
			
	for item in tmpList:
		item = item.rstrip();         #removes any Windows "\r"
		if (len(item) > 0):
			outList.append(item)
	return outList
#endDef

#------------------------------------------------------------------------------------------
# Update Application Ear
#------------------------------------------------------------------------------------------
def updateApp(applicationName,attr):
	try :
		print "Updating Application : ", applicationName
		AdminApp.update(applicationName, 'app', attr)
		print applicationName , " is updated successfully."
	except:
		print "Exception:Error occured while updating " + applicationName
		sys.stderr.write("Error occured while installing " + applicationName)
		raise
	#endTry :
#endDef


#-----------------------------------------------------------------
# Logs messages
#-----------------------------------------------------------------
def trace(message):
    print message;
#endDef

#------------------------------------------------------------------------------------------
# Update the applications
#------------------------------------------------------------------------------------------
def updateApplications(updateAppDict, appEarFolderLocation, nodeName, serverName, clusterName):
	isAppDeployed = False
	trace("************************************ Entering updateApplications ********************************")
	
	for app in updateAppDict.keys():
            appLocation = appEarFolderLocation+""+updateAppDict.get(app)            
            appAttrs = ["-operation", "update", "-contents", appLocation]

            if clusterName == '' :
                appAttrs.extend(["-node", nodeName, "-server", serverName])
            else:
	        	appAttrs.extend(["-cluster", clusterName])
            
            print "no cluster"
            updateApp(app, appAttrs)
	        
	trace("************************************ exiting updateApplications ********************************")
        



#------------------------------------------------------------------------------
# get the scope Id according to cellname ,nodename, servername
#------------------------------------------------------------------------------
def getScopeId(cellName,nodeName,serverName):
	trace("****************getScopeId() start****************");
	scopeId="";
	if (cellName ==''):
		scopeId='';
	elif (nodeName ==''):
		scopeId=AdminConfig.getid('/Cell:' + cellName);
	elif (serverName == ''):
		trace(" serverName empty, getting "+ cellName + ":" + nodeName);
		scopeId=AdminConfig.getid('/Cell:' + cellName + '/Node:' + nodeName);
	else:
		scopeId=AdminConfig.getid('/Cell:' + cellName + '/Node:' +nodeName + '/Server:' + serverName);
	#endIf
	trace("****************getScopeId() end****************");
	return scopeId;
#endDef

#------------------------------------------------------------------------------
# check whether the specified application is installed 
#------------------------------------------------------------------------------
def checkApplicationExist(appName, scope):
	trace("**************** checkApplicationExist(" + appName + ") start ****************");
	result = False
	applications = AdminApp.list(scope).split()
	for application in applications:
		if (application == appName):
			result = True
			trace("**************** checkApplicationExist() end:" + str(result) + " ****************");
			return result
	trace("**************** checkApplicationExist() end:" + str(result) + " ****************");
	return result
#endDef

#-----------------------------------------------------------------
# Check if Fabric has been configured:
#-----------------------------------------------------------------
def checkIfFabricProfile(scope):
	return checkApplicationExist("Fabric_Catalog", scope)


		
#--------------------------------------------------------------------------------------------------------------
#     MAIN     
#---------------------------------------------------------------------------------------------------------------
def main():
	clusterName = System.getProperty("bpm.upgrade.cluster.name")
	appName  = "isclite"
        adminModule = "wbsfportlet.war"
        contextRoot = "/ibm/wbsfportlet"
	
	if clusterName == "":
		clusterName = None
	#endif
	
	
	nodeName = System.getProperty("bpm.upgrade.node.name")
	cellName = System.getProperty('local.cell')
	
	#---------------------------------------------------------------------------------------------------------------------------------
	# Refresh Fabric EARs
	#---------------------------------------------------------------------------------------------------------------------------------
	    
	appEarFolderLocation = System.getProperty("was.install.root") + "/installableApps/"
	updateAppDict = {'Fabric_Catalog':'catalog.ear', 'Fabric_Tools':'tools.ear', 'Fabric_Trace':'trace.ear'}
	
	widgetPath = System.getProperty("was.install.root") + "/BusinessSpace/" +"widgets/" + "WBSF/" + "Widgets_Fabric.zip"

	if clusterName:
		trace("INFO: clusterName = " + clusterName + ".")
		scopeAtt = 'WebSphere:cell=' + cellName + ',cluster=' + clusterName		
		if (not checkIfFabricProfile(scopeAtt)):
				trace("No Fabric configured in target cluster, exit Fabric profile update in target cluster");
				return;
		
		deployAdminModule(appName,adminModule,java.lang.System.getProperty('was.install.root'),nodeName,contextRoot)
		updateApplications(updateAppDict, appEarFolderLocation, '', '', clusterName);
		updateOptions = ["-clusterName",clusterName,"-widgets",widgetPath,"-save true"]
		AdminTask.updateBusinessSpaceWidgets(updateOptions)
	
	else :
		trace("INFO: no cluster")		
		nodeId = getScopeId(cellName, nodeName, '')
		print "nodeId is: " + str(nodeId)
		serverList = wsadminToList(AdminConfig.list('Server', nodeId))
		for server in serverList:		
			serverName = AdminConfig.showAttribute(server, 'name')
			trace('serverName = ' + str(serverName))
			scopeAtt = 'WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName			
			if (not checkIfFabricProfile(scopeAtt)):
				trace("No Fabric configured in target server, exit Fabric profile update in target server");
				continue;
			deployAdminModule(appName,adminModule,java.lang.System.getProperty('was.install.root'),nodeName,contextRoot)
			updateApplications(updateAppDict, appEarFolderLocation, nodeName, serverName, '');
			updateOptions = ["-serverName",serverName,"-nodeName",nodeName,"-widgets",widgetPath,"-save true"]
			AdminTask.updateBusinessSpaceWidgets(updateOptions)

	
	AdminConfig.save()
	
	print "Synchronizing changes with all the nodes in cluster."
	
	#-----------------------------------------------------------------------------------------------------------------------------------
	#Sync node
	#------------------------------------------------------------------------------------------------------------------------------------
	
#	nodeList = AdminConfig.list('Node').splitlines()
#	for n in nodeList:
#		nodeid = n
#		i = nodeid.find('(')
#		nodename = nodeid[0:i]
#		objn = "type=NodeSync,node=" + nodename + ",*"
#		try :
#			Sync = AdminControl.completeObjectName(objn)
#			if Sync != "":
#				AdminControl.invoke(Sync, 'sync')
#				print "Done with synchronization for node " + nodename
#			continue
#		except :
#			print "AdminControl not available.Skipping " + nodename + " synchronization."
#		#endTry
#	#endFor
#	#Sync code complete
	
#-----------------------------------------------------------------
# Start Main function
#-----------------------------------------------------------------
main();
#-----------------------------------------------------------------
# End Main function
#-----------------------------------------------------------------